import { r as registerInstance, h, H as Host, g as getElement } from './core-07ed2030.js';
import { C as Coordinate } from './coordinate-2242d139.js';
function getPoints(event) {
    if (event instanceof TouchEvent) {
        return Array.from(event.touches).map(function (t) { return Coordinate.fromEvent(t); });
    }
    else {
        return [Coordinate.fromEvent(event)];
    }
}
var PanZoom = /** @class */ (function () {
    function class_1(hostRef) {
        var _this = this;
        registerInstance(this, hostRef);
        /**
         * Bounds used to define how far the content can be moved.
         */
        this._limits = { x: 0, y: 0, width: 1, height: 1 };
        /**
         * The initial scale of the content.  Any further scaling is done relative
         * to this value.  This is needed so content can be scaled to fit the size
         * of the `PanZoom` component.
         */
        this._initScale = 1;
        /**
         * The offset from the origin to move the object.
         */
        this._d = new Coordinate(0, 0);
        /**
         * The scale factor applied to the object.
         */
        this._scale = 1;
        /**
         * The number of active pointer events.
         */
        this._active = 0;
        /**
         * All the old pointer event locations.
         */
        this._lastPoints = [];
        /**
         * If true, the content of the `PanZoom` will be scaled so that it fills
         * the parent container initially.
         */
        this.scaled = false;
        /**
         * If true, the content of the `PanZoom` will be able to move beyond the
         * bounds of the parent container.
         */
        this.unbound = false;
        /**
         * The smallest scale factor allowed when scaling the content.
         */
        this.minScale = 0.5;
        /**
         * The largest factor allowed when scaling the content.  Calculated as a
         * factor of the original size.  So a value of `3` would limit the scale to
         * three times the original size.
         */
        this.maxScale = 3;
        /**
         * Handle when the mouse or a touch goes outside the bounds of the `PanZoom`
         * component.
         */
        this.onPointerLeave = function (event) {
            _this._active--;
            _this._lastPoints.pop();
            if (_this._active > 0) {
                _this._lastPoints = getPoints(event);
                return;
            }
            _this.root.removeEventListener('mousemove', _this.onPointerMove);
            _this.root.removeEventListener('mouseup', _this.onPointerUp);
            _this.root.removeEventListener('touchmove', _this.onPointerMove);
            _this.root.removeEventListener('touchend', _this.onPointerUp);
            _this.root.removeEventListener('mouseleave', _this.onPointerLeave);
            _this.root.removeEventListener('touchcancel', _this.onPointerLeave);
        };
        /**
         * Handle when a mouse button is pressed or a touch event starts.
         */
        this.onPointerDown = function (event) {
            if (event instanceof MouseEvent && event.button !== 0) {
                return;
            }
            _this._lastPoints = getPoints(event);
            _this._active++;
            if (_this._active === 1) {
                _this.root.addEventListener('mousemove', _this.onPointerMove);
                _this.root.addEventListener('mouseup', _this.onPointerUp);
                _this.root.addEventListener('touchmove', _this.onPointerMove, { passive: true });
                _this.root.addEventListener('touchend', _this.onPointerUp);
                _this.root.addEventListener('mouseleave', _this.onPointerLeave);
                _this.root.addEventListener('touchcancel', _this.onPointerLeave);
            }
        };
        /**
         * Handle when a touch or mouse moves.
         */
        this.onPointerMove = function (event) {
            if (_this._tEl === undefined) {
                return;
            }
            var points = getPoints(event);
            var avgPoint = points.reduce(Coordinate.midpoint);
            var avgLastPoint = _this._lastPoints.reduce(Coordinate.midpoint);
            var bounds = _this._tEl.getBoundingClientRect();
            _this._d.translate(Coordinate.difference(avgPoint, avgLastPoint));
            if (!_this.unbound) {
                _this._d.limit(_this._limits);
            }
            _this._lastPoints = points;
            /**
             * Multiple points only exist with touches and should be used to scale the
             * content.
             */
            if (points.length > 1) {
                var scaleDiff = Coordinate.squareDistance(points[0], points[1]) /
                    Coordinate.squareDistance(_this._lastPoints[0], _this._lastPoints[1]);
                if (_this._scale * scaleDiff < 0.5 || _this._scale * scaleDiff > 3) {
                    return;
                }
                _this._scale *= scaleDiff;
                _this._d.translate(Coordinate.difference(Coordinate.fromRectTopLeft(bounds), avgPoint)).scale(scaleDiff - 1);
                if (!_this.unbound) {
                    _this._d.limit(_this._limits);
                }
            }
        };
        /**
         * Handle when a touch ends or a mouse button is released to end the pan.
         */
        this.onPointerUp = function (event) {
            _this._active--;
            _this._lastPoints.pop();
            if (_this._active > 0) {
                _this._lastPoints = getPoints(event);
                return;
            }
            _this.root.removeEventListener('mousemove', _this.onPointerMove);
            _this.root.removeEventListener('mouseup', _this.onPointerUp);
            _this.root.removeEventListener('touchmove', _this.onPointerMove);
            _this.root.removeEventListener('touchend', _this.onPointerUp);
            _this.root.removeEventListener('mouseleave', _this.onPointerLeave);
            _this.root.removeEventListener('touchcancel', _this.onPointerLeave);
        };
        /**
         * Handle a `WheelEvent` to scale the contents.
         */
        this.onWheel = function (event) {
            if (_this._tEl === undefined) {
                return;
            }
            var bounds = _this._tEl.getBoundingClientRect();
            var delta = -event.deltaY;
            if (event.deltaMode === 1) {
                delta *= 15;
            }
            delta = Math.max(Math.min(delta, 60), -60);
            var scaleDiff = (delta / 300) + 1;
            if (_this._scale * scaleDiff < _this._initScale * _this.minScale ||
                _this._scale * scaleDiff > _this._initScale * _this.maxScale) {
                return;
            }
            _this._scale *= scaleDiff;
            _this._d.translate(Coordinate.difference(Coordinate.fromRectTopLeft(bounds), Coordinate.fromEvent(event)).scale(scaleDiff - 1));
            if (!_this.unbound) {
                _this._computeLimits();
                _this._d.limit(_this._limits);
            }
        };
    }
    /**
     * Lifecycle event called when the component has been loaded and added to
     * the DOM tree.
     */
    class_1.prototype.componentDidLoad = function () {
        this.root.addEventListener('mousedown', this.onPointerDown);
        this.root.addEventListener('touchstart', this.onPointerDown, { passive: true });
        this.root.addEventListener('wheel', this.onWheel);
        if (this.scaled) {
            this._setInitialScale();
        }
        if (!this.unbound) {
            this._computeLimits();
        }
    };
    /**
     * Computes the limits used to constrain how far the content can move.
     */
    class_1.prototype._computeLimits = function () {
        var content = this.root.querySelector('[slot=pz-content]');
        if (content !== null) {
            if (this._initSize === undefined) {
                var contentBounds = content.getBoundingClientRect();
                this._initSize = {
                    height: contentBounds.height,
                    width: contentBounds.width,
                };
            }
            var parentBounds = this.root.getBoundingClientRect();
            var contentW = this._initSize.width * this._scale;
            var contentH = this._initSize.height * this._scale;
            this._limits = new DOMRect(Math.min(0, parentBounds.width - contentW), // left / x
            Math.min(0, parentBounds.height - contentH), // top / y
            Math.abs(contentW - parentBounds.width), // width
            Math.abs(contentH - parentBounds.height) // height
            );
        }
    };
    /**
     * Compute the initial scale used to resize the content to best fit within
     * the bounds of the `PanZoom` component.
     */
    class_1.prototype._setInitialScale = function () {
        var content = this.root.querySelector('[slot=pz-content]');
        if (content !== null) {
            var contentBounds = content.getBoundingClientRect();
            var parentBounds = this.root.getBoundingClientRect();
            this._initScale = this._scale = Math.max(parentBounds.width / contentBounds.width, parentBounds.height / contentBounds.height);
        }
    };
    /**
     * Lifecycle method called when the component needs to redraw.
     */
    class_1.prototype.render = function () {
        var _this = this;
        var style = {
            'transform': "translate3d(" + this._d.x + "px, " + this._d.y + "px, 0px) scale(" + this._scale + ")",
        };
        return (h(Host, { class: "rl-pan-zoom" }, h("div", { class: "rl-pan-zoom__transform", style: style, ref: function (el) { return _this._tEl = el; } }, h("slot", { name: "pz-content" }))));
    };
    Object.defineProperty(class_1.prototype, "root", {
        get: function () { return getElement(this); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(class_1, "style", {
        get: function () { return ".rl-pan-zoom__transform,:host{display:block;position:absolute;top:0;right:0;bottom:0;left:0}.rl-pan-zoom__transform{-webkit-transform-origin:0 0;transform-origin:0 0}"; },
        enumerable: true,
        configurable: true
    });
    return class_1;
}());
export { PanZoom as rl_pan_zoom };
